/* --- WVHEADER.C ---------------------------------------------
 *
 *  This file contains the code necessary to create the initial skeleton
 *  version of an article, which will be edited by the user.
 *
 *  Mark Riordan   24-JAN-1990
 * 
 *  Major overhaul by John S. Cooper  October 1994 
 */


/*
 *
 * $Id: wvheader.c 1.27 1994/11/02 01:50:54 jcooper Exp $
 * $Log: wvheader.c $
 * Revision 1.27  1994/11/02  01:50:54  jcooper
 * lots of changes for .93.0
 * 
 * Revision 1.26  1994/09/16  00:42:53  jcooper
 * cc-by-mail feature and cleanup for 92.6
 * 
 * Revision 1.25  1994/08/24  18:40:15  mrr
 * authorization enables post/mail
 *
 * Revision 1.24  1994/05/27  01:29:29  rushing
 * unnecessary winundoc.h
 *
 * Revision 1.23  1994/02/24  21:31:49  jcoop
 * jcoop changes
 *
 * Revision 1.22  1994/02/09  18:01:08  cnolan
 * cnolan 90.2 changes
 *
 * Revision 1.21  1994/01/18  09:51:34  jcoop
 * Use new version number scheme on X-Mailer header line
 *
 * Revision 1.20  1994/01/15  20:31:16  jcoop
 * Font/color stuff and signature file handling
 * 
 * Revision 1.19  1993/12/08  01:27:21  rushing
 * new version box and cr lf consistency
 * 
 * Revision 1.18  1993/08/25  18:53:17  mbretherton
 * MRB merge, mail & post logging
 *
 *
 * remove underscore for BC++ compile   Bretherton
 *
 * Revision 1.17  1993/08/17  21:53:06  UNKNOWN
 * Updated version to Version 0.82 (JD)
 *
 * Revision 1.16  1993/07/13  22:11:31  rushing
 * strncpy crashing in parseaddress
 *
 * Revision 1.15  1993/07/06  21:09:50  cnolan
 * win32 support
 *
 * Revision 1.14  1993/06/29  20:06:07  rushing
 * use localtime for Date header
 *
 * Revision 1.13  1993/06/29  00:22:13  rushing
 * correct rfc822/rfc1036 MakeArtHeaderDate function
 *
 * Revision 1.12  1993/06/28  17:51:21  rushing
 * warnings
 *
 * Revision 1.11  1993/06/22  19:08:40  rushing
 * remove warnings
 *
 * Revision 1.10  1993/06/22  16:46:33  bretherton
 * robust parse_address
 *
 * Revision 1.9  1993/06/11  01:05:17  rushing
 * new version number
 *
 * Revision 1.8  1993/06/11  00:11:26  rushing
 * second merge from Matt Bretherton sources
 *
 *
 * Revision 1.7  1993/05/24  23:34:12  rushing
 * further refinements to header manipulation
 * extend reuse of header formatting capabilities
 *
 *
 * Revision 1.5  1993/05/18  22:10:45  rushing
 * smtp support
 *
 * Revision 1.4  1993/05/06  19:44:10  rushing
 * CompareStringNoCase was using _tolower rather than tolower.
 * according to c70 docs, _tolower is undefined for letters that
 * are not uppercase.  Yikes!
 *
 * Revision 1.3  1993/04/30  21:17:01  rushing
 * put the X-Newsreader: line back in.
 *
 * Revision 1.2  1993/03/30  20:07:37  rushing
 * MAPI
 *
 * Revision 1.1  1993/02/16  20:54:22  rushing
 * Initial revision
 *
 *
 */

#include <windows.h>
#include <windowsx.h>
#include "wvglob.h"
#include "winvn.h"
#pragma hdrstop
#include <ctype.h>
#include <time.h>
#include <stdlib.h>

static char * days[] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
static char * months[] = { "Jan","Feb","Mar","Apr","May","Jun",
				"Jul","Aug","Sep","Oct","Nov","Dec"};


BOOL AuthenticatePosting (BOOL AuthReq);
/*--- function CreateComposeWnd -----------------------------------------
 *
 *    Create the window for composing the text of a posting,
 *    if it's OK to post.
 *
 *    Entry    Hwnd     Handle to parnet window
 *             Doc      points to the document to which we are posting
 *                      a followup--NULL if it's a new posting.
 *
 *    Exit     Returns the handle of the newly-created window
 *              (zero if failure).
 */
HWND
CreateComposeWnd (HWND hWnd, TypDoc *Doc, int DocType)
{
	register int i;
	BOOL found;
	BOOL AuthReq;
	unsigned int width;
	char *TitlePtr, *WndClassName;
	int maxNum;
	WndEdit *wndEdits;
	  
	if (DocType == DOCTYPE_POSTING) {
		wndEdits = WndPosts;
		maxNum = MAXPOSTWNDS;

		AuthReq = AuthReqPost;
		WndClassName = "WinVnPost";
		if (Doc) {
			TitlePtr = "Followup Article";
		} else {
			TitlePtr = "New Article";
		}
	} else {         
		wndEdits = WndMails;
		maxNum = MAXMAILWNDS;

		AuthReq = AuthReqMail;
		WndClassName = "WinVnMail";
		if (Doc) {
			if (DocType == DOCTYPE_FORWARD)
				TitlePtr = "Forward Message";
			else	
				TitlePtr = "Reply Message";
		} else {
			TitlePtr = "New Mail Message";
			strcpy(AddressString,"");
		}
	}

	for (i = 0, found = FALSE; !found && i < maxNum; i++) {
		if (!wndEdits[i].hWnd) {
			found = TRUE;
			break;
		}
	}

	if (!found) {
		MessageBox(hWnd, "Too many composition windows are open.\nPlease send or close one before continuing.",
			"Cannot Continue", MB_OK|MB_ICONSTOP);
	} else if (AuthenticatePosting (AuthReq)) {
		if (xScreen > 78 * ArtCharWidth) {
			width = 78 * ArtCharWidth;
		} else {
			width = xScreen - 2 * ArtCharWidth;
		}
	
		wndEdits[i].Doc = Doc;
		wndEdits[i].composeType = DocType;
		wndEdits[i].hWnd = CreateWindow (WndClassName, TitlePtr,
				WS_OVERLAPPEDWINDOW /* | WS_VSCROLL */,
				i * CharWidth,
				(int) (yScreen * 3 / 8) + (1 + i) * LineHeight,
				(int) width,
				(int) (yScreen * 5 / 8) - (2 * LineHeight),
				NULL, NULL, hInst,
				&wndEdits[i]);
		
	}
	return (wndEdits[i].hWnd);
}

/*--- function GetHeaderLine -------------------------------------------
 *
 *  Given a document, get a line from the header portion of that document
 *  whose prefix matches a given prefix.  Prefix = first word in line.
 *  For instance, you might call this routine to say "Get the 'Subject:'"
 *  line from this document.
 *
 *    Entry    Doc      points to the document whose header we are scanning.
 *                      The header is all the lines up to the first blank line.
 *             Prefix   is the character string which will identify the
 *                      line we are seeking.  It is the first word
 *                      (blank-delimited) in a line in the header.
 *             BufLen   is the number of bytes left in the buffer Buf.
 *
 *    Exit     Returns TRUE iff we returned a line.
 *             Buf      if line was returned, contains that line, zero-
 *                      terminated.
 */
BOOL
GetHeaderLine (TypDoc *Doc, char *Prefix, char *Buf, int BufLen)
{
	char *bufptr = Buf;
	TypBlock far *BlockPtr;
	TypLine far *LinePtr;
	unsigned int Offset;
	HANDLE hBlock;
	TypLineID MyLineID;
	int found = FALSE;
	
	TopOfDoc (Doc, &BlockPtr, &LinePtr);
	while (ExtractTextLine (Doc, LinePtr, Buf, BufLen)) {
		/* Is this a blank line signifying the end of the header?      */
		if (IsBlankStr(Buf))
			break;
		
		if (_strnicmp (Buf, Prefix, strlen (Prefix)) == 0) {
			found = TRUE;
			break;
		}
		if (!NextLine (&BlockPtr, &LinePtr))
			break;
	}
	UnlockLine (BlockPtr, LinePtr, &hBlock, &Offset, &MyLineID);
	return (found);
}

/*--- function NextToken -----------------------------------------------
 *
 *  Position a pointer to the next token in a string.
 *  Delimiters are space and tab.
 *
 *    Entry    cptr     points to a position in a zero-terminated string.
 *
 *    Exit     Returns TRUE iff a next token was found.
 *             cptr     points to the next token, if found--else
 *                      it is unchanged.
 */
BOOL
NextToken (char **cptr)
{
  /* Skip to end of current token, if any. */
  while (**cptr != ' ' && **cptr != '\t' && **cptr)
    (*cptr)++;

  /* Skip past white space. */
  while (**cptr && (**cptr == ' ' || **cptr == '\t'))
    (*cptr)++;
  
  return (**cptr != '\0');
}

/*--- function AppendTextToEditBuf ----------------------------------------
 *
 *    Appends a zero-terminated text line to a buffer to be given
 *    to an Edit window.  Used in building messages to be displayed
 *    and edited by an Edit window.
 *
 *    Entry    instr    points to a text line to add.  It is terminated
 *                      by a zero byte and does not end in CR or LF.
 *             left     is the number of characters left in buf.
 *
 *    Exit     buf      contains the line, terminated by CR and LF.
 *                      buf now points to the next available byte.
 *             left     has been decremented as appropriate.
 */
void
AppendTextToEditBuf (char *instr, char far **buf, long *left)
{
  while (--(*left) > 0 && *instr)
    {
      *((*buf)++) = *(instr++);
    }
  if (--(*left) > 0)
    *((*buf)++) = '\r';

  if (--(*left) > 0)
    *((*buf)++) = '\n';

  if ((*left) > 0)
    **buf = '\0';
}

/*--- function ParseAddress --------------------------
 *   Scan the
 *       address field (null terminated ascii string)
 *  representing any email address and optional
 *       name
 *
 *   and extract
 *   addressout which is the email address
 *   nameout    which is the name (enclosed in parenthises)
 *
 */
void 
ParseAddress(char *headerline, char *addressout, long addressoutlen, char *nameout, long nameoutlen)
{ 
  char *ptra = headerline , *ptrb, *ptrc ;
  char *iptr , *optr;
  int  lastspace ;
  long int adlen = addressoutlen - 1 ,nmlen = nameoutlen -1 ;
  long int l;

   *nameout = *addressout = '\0';

// Note parsing not strictly correct as anything in quotes '"'
// overrides the meaning of lexical constructs '<' and '('

// Seems this works and is robust but is now in the main
// group list read and should be extremely efficient
// suggest a single pass 'stream' parse more appropriate
// but is more work

  if (ptrb=strchr(headerline,'<'))
  {  // address enclosed in brackets  
     l=min(ptrb-ptra-1,nmlen) ;
     if (l < 0) l = 0;     /* this was killing print??? */
     if (l>0)
     {                      // otherwise strncpy crashes
        nmlen-=l ;
        strncpy(nameout,ptra,(int)l) ;
        nameout[l]=0;
     }
     if(  ptrc=strchr(headerline,'>') )
     {
   if (nmlen >0)
        {
      strncpy(nameout+l,ptrc+1,(int)nmlen) ;
      nameout[nameoutlen-1]=0 ;
   }
   l=min(ptrc-ptrb-1,adlen) ;
     } else
     {
   l=adlen ;
     }
     if (l>0)
     {
     strncpy(addressout,ptrb+1,(int)l);
     addressout[l]=0;
     }

  } else
    if (ptrb=strchr(headerline,'('))
  {  // name enclosed in braces
     l=min(ptrb-ptra-1,adlen) ;
     if (l>0)
     {
        adlen-=l ;
   strncpy(addressout,ptra,(int)l) ;
        addressout[l+1]=0;
     }
     if( ptrc=strchr(headerline,')'))
     {
   if (adlen > 0)
        {
      strncpy(addressout+l,ptrc+1,(int)adlen) ;
      addressout[addressoutlen-1]=0 ;
        }
   l=min(ptrc-ptrb-1,nmlen) ;
     } else
     {
   l=nmlen ;
     }
     if (l>0)
     {
     strncpy(nameout,ptrb+1,(int)l);
     nameout[l]=0;
     }      

  } else
  { // just an address
     strncpy(addressout,ptra,(int)adlen) ;
     addressout[addressoutlen-1]=0 ;
  }

// Clean Name   - remove multiple space and quotes
  lastspace=TRUE ;
  for(iptr=optr=nameout ; *iptr ; iptr++)
  {  switch( *iptr)
     {
       case ' ':
    if (!lastspace) *optr++=*iptr ;
    lastspace=TRUE ;
         break;
       case '"':    
    break ;
       default :
    *optr++=*iptr ;
    lastspace=FALSE ;
      }
   }
   *optr=0 ;

// Clean Address
  for(iptr=optr=addressout ; *iptr ; iptr++)
  {  switch( *iptr)
     {
       case ' ':
         break;
       default :
    *optr++=*iptr ;
    lastspace=FALSE ;
      }
   }
   *optr=0 ;

   if (*nameout==0)
   {  // if no name default to address
      strncpy(nameout,addressout,(int)nmlen) ;
      addressout[nameoutlen-1]=0 ;
   }

 }

/*------------------------------------------------------------------------
 * Getxxx functions
 * Fill a buffer with appropriate string for use in header
 * possibly based on a refDoc (doc to which we are replying/following-up)
 * jsc 10/3/94
 */           
BOOL
GetToAddress(char *buf, int len, TypDoc *refDoc, HWND hWnd)
{
	char headerLine[MAXHEADERLINE];
	char *ptr = headerLine ;
	BOOL gotwho;
	char replyAddress[MAXHEADERLINE], replyName[MAXHEADERLINE];
		
	*buf = '\0';
	if (refDoc) {
		if (GetHeaderLine (refDoc, "From:", headerLine, MAXHEADERLINE)) {
			NextToken (&ptr);
			ParseAddress (ptr, AddressString, MAXHEADERLINE, NameString, MAXHEADERLINE) ;
		}
		if (gotwho = GetHeaderLine (refDoc, "Reply-To:", headerLine, MAXHEADERLINE)) {
			NextToken (&ptr);
			ParseAddress (ptr, replyAddress, MAXHEADERLINE, replyName, MAXHEADERLINE) ;
		}
		if (gotwho && strcmp (AddressString, replyAddress)) {
			if (*AddressString && ConfirmReplyTo)
			{ 
				sprintf (str, "Use Reply-To: (%s) instead of\nFrom: (%s) in reply?", 
					replyAddress, AddressString);
				                      
				if (MessageBox (hWnd, str, "Confirm Reply-To", 
					MB_YESNO|MB_ICONQUESTION) == IDNO) {
					gotwho = FALSE;
				}
			}
			if (gotwho) {
				strcpy (AddressString, replyAddress);
			}
		}
		if (*AddressString) {
			strntcpy(buf, AddressString, len);
			return TRUE;
		}
	}
	return FALSE;
}

BOOL
GetNewsgroups(char *buf, int len, TypDoc *refDoc)
{
	char headerLine[MAXHEADERLINE];
	char *ptr = headerLine;
	BOOL gotnews = FALSE;

	if (refDoc) {
		gotnews = GetHeaderLine (refDoc, "Newsgroups:", headerLine, MAXHEADERLINE);
		if (gotnews) {
			NextToken (&ptr);
			strntcpy(buf, ptr, len);
			return TRUE;
		}
	}
	if (NewsgroupsPtr && *NewsgroupsPtr) {
		strntcpy(buf, NewsgroupsPtr, len);
		return TRUE;
	}
	return FALSE;
}

BOOL
GetFromAddress(char *buf, int len, TypDoc *refDoc)
{
	*buf = '\0';
	if (*MailAddress) {
		strntcpy(buf, MailAddress, len);
		if (UserName[0]) {
			len -= strlen(MailAddress);
			_snprintf(str, len, " (%s)", UserName);
			strcat(buf, str);
		}
		return TRUE;
	}
	return FALSE;
}

BOOL
GetSubject(char *buf, int len, TypDoc *refDoc, int docType)
{
	char headerLine[MAXHEADERLINE];
	char *ptr = headerLine;
		
	*buf = '\0';
	if (refDoc) {
		if (GetHeaderLine (refDoc, "Subject:", headerLine, MAXHEADERLINE)) {
			NextToken (&ptr);
			if (docType == DOCTYPE_FORWARD) {
				_snprintf(buf, len, "%s (fwd)", ptr);
			} else {
				if (_strnicmp (ptr, "re:", 3)) {
					_snprintf(buf, len, "Re: %s", ptr);
				} else {
					strntcpy(buf, ptr, len);
				}
			}
			return TRUE;
		}
	}
	return FALSE;
}

BOOL      
GetCcAddress(char *buf, int len, TypDoc *refDoc)
{
	char from[MAXHEADERLINE], cc[MAXHEADERLINE];
	char *ptr = from;

	*buf = '\0';
	if (refDoc) {
		if (GetHeaderLine (refDoc, "From:", from, MAXHEADERLINE-1)) {
			NextToken (&ptr);
			ParseAddress (ptr, cc, MAXDIALOGSTRING , str, MAXINTERNALLINE );
			strntcpy(buf, cc, len);
			return TRUE;
		}
    }
	return FALSE;
}

	
BOOL
GetReplyToAddress(char *buf, int len, TypDoc *refDoc)
{
	*buf = '\0';
	if (*ReplyTo && strcmp(ReplyTo, MailAddress)) {
		strntcpy(buf, ReplyTo, len);
		return TRUE;
	}
	return FALSE;
}

BOOL
GetOrganization(char *buf, int len, TypDoc *refDoc)
{
	*buf = '\0';
	if (*Organization) {
		strntcpy(buf, Organization, len);
		return TRUE;
	}
	return FALSE;
}


BOOL
GetHeaderContents(char *buf, int len, TypDoc *refDoc, char *str)
{
	char headerLine[MAXHEADERLINE];
	char *ptr = headerLine;
	long int BufLen = MAXDIALOGSTRING ;
		
	*buf = '\0';
	if (refDoc) {
		if (GetHeaderLine (refDoc, str, headerLine, MAXHEADERLINE)) {
			NextToken (&ptr);
			strntcpy(buf, ptr, len);
			return TRUE;
		}
	}
	return FALSE;
}  

/* If refDoc has a references then use that followed by one space. 
 * Then add the refDoc message-id.
 */
BOOL
GetReferences(char *buf, int len, TypDoc *refDoc)
{
	char headerLine[MAXHEADERLINE];
	char *ptr;
	int tokenLen, lenToGo = len;
		
	*buf = '\0';
	if (refDoc) {
		if (GetHeaderLine (refDoc, "References:", headerLine, MAXHEADERLINE)) {
			ptr = headerLine;
			NextToken (&ptr);
			tokenLen = strlen(ptr);
			strncpy(buf, ptr, len);
			buf[tokenLen++] = ' ';
			buf[tokenLen] = '\0';
			lenToGo -= tokenLen;
		}
		if (GetHeaderLine (refDoc, "Message-ID:", headerLine, MAXHEADERLINE)) {
			ptr = headerLine;
			NextToken (&ptr);
			strncat(buf, ptr, lenToGo);
		}
	}
	return (*buf != '\0');
}

/* For mail, an In-Reply-To field contains the msg-id of the msg 
 * being replied to.
 */
BOOL
GetInReplyTo(char *buf, int len, TypDoc *refDoc)
{
	char headerLine[MAXHEADERLINE];
	char *ptr;
		
	*buf = '\0';
	if (refDoc) {
		if (GetHeaderLine (refDoc, "Message-ID:", headerLine, MAXHEADERLINE)) {
			ptr = headerLine;
			NextToken (&ptr);
			strntcpy(buf, ptr, len);
		}
	}
	return (*buf != '\0');
}

/* Returned date str is computed from operating system time.
 * Be sure to set the TZ environment variable correctly for
 * "gmtime" to work properly.  Typical setting:  SET TZ=EST5
 */
BOOL
GetDate(char *buf, int len)
{
	struct tm *timeptr;
	time_t curtime;
	
	if (!getenv("TZ")) {
		MessageBox (NetDoc.hDocWnd,
			"Timezone environment variable 'TZ' not set.\n"
			"You must set this variable for your messages\n"
			"to be timestamped correctly.\n"
			"For example, you could add the following to your\n"
			"'autoexec.bat' file:\n"
			"set TZ=EST5EDT\n",
			"Warning:",
			MB_OK|MB_ICONHAND);
	}
	
	curtime = time ((time_t *) NULL);
	timeptr = localtime (&curtime);
	  
	/* rfc1036&rfc822 acceptable format */
	/* Mon, 29 Jun 93 02:15:23 GMT */
	_snprintf (buf, len, "%s, %.2d %s %.2d %.2d:%.2d:%.2d %s", 
		days[timeptr->tm_wday],
		timeptr->tm_mday,
		months[timeptr->tm_mon],
		timeptr->tm_year,
		timeptr->tm_hour,
		timeptr->tm_min,
		timeptr->tm_sec,
		tzname[0]);
	
	return TRUE;
}


/* Returned date str is computed from operating system time.
 * Be sure to set the TZ environment variable correctly for
 * "gmtime" to work properly.  Typical setting:  SET TZ=EST5
 */
BOOL
GetMailboxDate(char *buf, int len)
{
	struct tm *timeptr;
	time_t curtime;
	
	curtime = time ((time_t *) NULL);
	timeptr = localtime (&curtime);
	  
	/* mailbox acceptable format */
	/* i.e. Tue 15 Mar 1994 11:11:22 */
	_snprintf (buf, len, "%s %.2d %s %.2d %.2d:%.2d:%.2d", 
		days[timeptr->tm_wday],
		timeptr->tm_mday,
		months[timeptr->tm_mon],
		timeptr->tm_year,
		timeptr->tm_hour,
		timeptr->tm_min,
		timeptr->tm_sec);
	
	return TRUE;
}

/* ------------------------------------------------------------------------
 * Generate 'says' line, given orig doc, template, and a dest buffer
 *  Replace %i with article ID
 *  Replace %d article date
 *  Replace %a with author email address
 *  Replace %n with author full name
 *
 * called by MakeBodyContents
 */
void
GenerateSaysLine (char *line, TypDoc *Doc, char *template)
{
	register char *src, *dest, *ptr;
	char mesId[MAXHEADERLINE];
	char from[MAXHEADERLINE];
	char date[MAXHEADERLINE];
	char address[MAXDIALOGSTRING];
	char name[MAXDIALOGSTRING];
	char *mesPtr = mesId;
	char *fromPtr = from;
	char *datePtr = date;
	char *addrPtr = address;
	char *namePtr = name;
	BOOL gotMesId, gotFrom, gotDate;

	gotMesId = GetHeaderLine (Doc, "Message-ID:", mesId, MAXHEADERLINE);
	if (gotMesId)
		NextToken (&mesPtr);
	else
		mesPtr = "<Unknown>";

	gotDate = GetHeaderLine (Doc, "Date:", date, MAXHEADERLINE);
	if (gotDate)
		NextToken (&datePtr);
	else
		datePtr = "<Unknown>";

	*address = *name = '\0';
	gotFrom = GetHeaderLine (Doc, "From:", from, MAXHEADERLINE);
	if (gotFrom) {
		NextToken (&fromPtr);
        ParseAddress (fromPtr, address, MAXDIALOGSTRING , name, MAXDIALOGSTRING);
		if (!(*addrPtr))
			addrPtr = "<Unknown>";
		if (!(*namePtr))
			namePtr = "<Unknown>";
			
	} else {
		fromPtr = "<Unknown>";
	}

	for (src = template, dest = line; *src != '\0';)
	{
		if (*src == '%')
		{
			src++;
			switch (*src)
			{
			case '%':		// literal percent is %%
				*dest++ = '%';
				break;
			case 'i':
				for (ptr = mesPtr; *ptr; *dest++ = *ptr++);
				break;
			case 'd':
				for (ptr = datePtr; *ptr; *dest++ = *ptr++);
				break;
			case 'n':
				for (ptr = namePtr; *ptr; *dest++ = *ptr++);
				break;
			case 'a':
				for (ptr = addrPtr; *ptr; *dest++ = *ptr++);
				break;

			default:
				*dest++ = '%';
				*dest++ = *src;
				break;

			}
			src++;	// skip control char after %
			continue;
		}
		*dest++ = *src++;	
	}
	*dest = '\0';   
}

/*--- function MakeBodyContents ----------------------------------------------
 *
 *    Make the body of the article.  This is null if there's no article
 *    to reply to, else it's text of the form:
 *    In article <Message-ID>, <user> says:
 *    >line 1
 *    >line 2
 *    > .....
 *
 *    Entry    Doc      points to the article being replied to, else
 *                      NULL if none.
 *             left     is the number of bytes left in MesBuf.
 *
 *    Exit     MesBuf   contains the added lines, and has been updated
 *                      to point to just after the last added byte.
 *             left     has been decremented appropriately.
 *             Return value is not used, for now.
 *
 * called by MakeBody
 */
BOOL 
MakeBodyContents (TypDoc *Doc, char far **MesBuf, long *left, int DocType)
{
	HANDLE hBlock;
	unsigned int Offset, quoteIndSize;
	TypLineID MyLineID;
	TypBlock far *BlockPtr;
	TypLine far *LinePtr;
	char line[MAXHEADERLINE];
	register unsigned long i;
  
	if (Doc) {
		TopOfDoc (Doc, &BlockPtr, &LinePtr);
		if (DocType == DOCTYPE_FORWARD) {
			AppendTextToEditBuf ("----Forwarded----", MesBuf, left); 
			/* include original headers in forwarded doc */
		} else {
			GenerateSaysLine(line, Doc, 
				(DocType==DOCTYPE_POSTING) ? FollowupSaysTemplate : ReplySaysTemplate);
			AppendTextToEditBuf (line, MesBuf, left);
	
			/* Skip past headers */
			while (ExtractTextLine (Doc, LinePtr, line, MAXHEADERLINE) &&
					!IsBlankStr(line)) {
				if (!NextLine (&BlockPtr, &LinePtr))
				break;
			}
		}
	
		/* Copy body of reply document into the body of this article,  */
		/* prepending a ">" to each line */
		
		/* following line commented after a merge of MRB code (SMR)*/
		/* line[0] = '>'; */
		
		if (DocType == DOCTYPE_FORWARD) {
			quoteIndSize = 0;
		} else {
			quoteIndSize = 1;
			line[0] = QuoteLineInd ;
		}
		while (ExtractTextLine (Doc, LinePtr, line + quoteIndSize, MAXHEADERLINE - 1)) {
			AppendTextToEditBuf (line, MesBuf, left);
			if (!NextLine (&BlockPtr, &LinePtr))
				break;
		}
		UnlockLine (BlockPtr, LinePtr, &hBlock, &Offset, &MyLineID);
	}

	if (EnableSig && Signature->numLines > 0) {
		AppendTextToEditBuf ("", MesBuf, left); /* blank line before sig */
		if (strcmp(TextBlockLine(Signature, 0), SIG_DELIMITER)) {
			AppendTextToEditBuf (SIG_DELIMITER, MesBuf, left);
		}
		for (i = 0; i < Signature->numLines; i++) {
			AppendTextToEditBuf (TextBlockLine(Signature, i), MesBuf, left);
		}
	}
	return TRUE;
}

/* MakeBody sets endBuf to an alloced buf containing body, or NULL if no body.
 * It is the caller's responsibility to free the buf.
 * This can fail.
 */
BOOL
MakeBody(char **endBuf, TypDoc *refDoc, int docType, HWND hWnd)
{
	char *ptr, *buf;
	long bytesLeft;
	
	/* Compute the number of bytes we need to hold a straight ASCII representation
	 * of the initial text of the reply, and allocate a buffer of that size.
	 */
	bytesLeft = 0L;
	if (EnableSig && Signature->numLines) {
		// the 2 * Signature->numLines is to count \r\n for each line
		bytesLeft += Signature->numBytes + 2 * Signature->numLines + strlen(SIG_DELIMITER) + 10;
	}
	if (refDoc) {
		bytesLeft += (2 + NumBlocksInDoc (refDoc)) * refDoc->BlockSize;
	} 
	if (bytesLeft) {
		if ((ptr = buf = (char *)GlobalAllocPtr(GMEM_MOVEABLE, bytesLeft * sizeof(char))) == NULL) {
			MessageBox (hWnd, "Failed to Generate Article", "Memory Allocation Failure", MB_OK|MB_ICONSTOP);
			return FAIL;
		}
		*ptr = '\0';
		MakeBodyContents (refDoc, &ptr, &bytesLeft, docType);
		*endBuf = buf;
	} else {
		*endBuf = (char *)NULL;
	}
	return SUCCESS;
}




/*--- function AuthenticatePosting --------------------------------
 *
 *   Determine whether the user has sent valid authentication
 *   information to the host.
 *
 *    Entry    Authenticated  is TRUE if we're already authorized.
 *             AuthReq        is TRUE if we need authorization.
 *
 *    Exit     Returns TRUE if we are authorized, else FALSE.
 *                                                             
 *    This routine is largely unimplemented.
 *    Intended
 *    Method   If we're already authorized, just return TRUE.
 *             Otherwise, make sure the comm channel to the server
 *             isn't busy.  If it is, put up a message to that
 *             effect and quit.
 *             If not busy, get authorization information from the
 *             user and present it to the server (in encoded form).
 *             Wait until the server responds or until the Abort
 *             Protocol flag is set (by the user via a menu).
 *             If authorized, return TRUE, else put up a
 *             message to the user and return FALSE.
 */
BOOL
AuthenticatePosting (BOOL AuthReq)
{  
   BOOL whetherOK=FALSE;
                      
   if(AuthReq && !Authenticated) {
      MessageBox(NetDoc.hDocWnd,
      "Sorry, you must identify yourself to the news server \
before sending.  Choose the Config/Configure Comm... menu option \
and enter your username and password.  \
Then Choose Network/Disconnect and Network/Connect.",
"No permission",MB_OK);
   } else {
      whetherOK = TRUE;
   }
   
   return(whetherOK);
}
