/*--- WVMAIL.C ------------------------------------------------------------
 *
 *    This file contains the window procedure for the windows
 *    used to edit outgoing mail messages.
 *
 *    Mark Riordan   11 June 1991
 *
 *    MAPI interface implemented by MAtthew Bretherton  March 1993
 *
 *    Much of the send/forward/etc code, as well as the main window handler
 *    now replaced by new code in wvcompos.c and wvattach.c (jsc 10/1/94)
 */


/*
 * $Id: wvmail.c 1.25 1994/11/02 01:51:09 jcooper Exp $
 * $Log: wvmail.c $
 * Revision 1.25  1994/11/02  01:51:09  jcooper
 * lots of changes for .93.0
 * 
 * Revision 1.24  1994/09/19  01:15:21  jcooper
 * ignore MAPI in auto-select if using win32s
 * 
 * Revision 1.23  1994/09/18  22:49:31  jcooper
 * support for cascading windows
 * 
 * Revision 1.22  1994/09/16  01:05:02  jcooper
 * General cleanup for 92.6
 * 
 * Revision 1.21  1994/08/24  18:40:15  mrr
 * authorization enables post/mail
 *
 * Revision 1.20  1994/05/27  01:29:29  rushing
 * unnecessary winundoc.h
 *
 * Revision 1.19  1994/05/27  01:18:47  rushing
 * new smtp
 *
 * Revision 1.18  1994/02/24  21:32:14  jcoop
 * jcoop changes
 *
 * Revision 1.17  1994/02/09  18:39:32  rushing
 * ':' missing on case
 *
 * Revision 1.16  1994/02/09  18:01:08  cnolan
 * cnolan 90.2 changes
 *
 * Revision 1.15  1994/01/24  17:40:28  jcoop
 * 90.2 changes
 *
 * Revision 1.14  1994/01/15  20:32:30  jcoop
 * Attachments, color/font, global editWnd, bugfix in WM_SEND
 *
 * Revision 1.13  1994/01/11  18:26:19  cnolan
 * from conor nolan (missed changes)
 * 
 * Revision 1.12  1993/12/08  01:28:38  rushing
 * new version box and cr lf consistency
 *
 * Revision 1.11  1993/08/25  18:54:36  mbretherton
 * MRB merge, mail & post logging
 *
 * Revision 1.10  1993/06/29  00:22:35  rushing
 * make smtp headers have Date: header
 *
 * Revision 1.9  1993/06/28  17:51:39  rushing
 * fixed compiler warnings
 *
 * Revision 1.8  1993/06/22  19:09:14  rushing
 * mail force type selection via dialog
 *
 * Revision 1.7  1993/06/22  16:45:51  bretherton
 * *** empty log message ***
 *
 * Revision 1.xx MBretherton - auto close mail window on
 *               successful dispatch of mail
 *               & tweak smtp/mapi cotnrol
 * Revision 1.6  1993/06/11  00:11:26  rushing
 * second merge from Matt Bretherton sources
 *
 * Revision 1.5  1993/05/19  23:49:08  rushing
 * notify user after succesfull forward_smtp_message
 *
 * Revision 1.4  1993/05/18  22:10:45  rushing
 * smtp support
 *
 * Revision 1.3  1993/04/27  21:29:23  rushing
 * added rcs log and id magikstrings
 *
 *
 */

#include <windows.h>
#include <windowsx.h>
#include "wvglob.h"
#include "winvn.h"
#pragma hdrstop

/*---- function MailLogout---------------------------------------------
 *
 *   Finish up the mail session
 *
 */
int
MailClose (HWND hWnd) 
{  
  return ((MailCtrl.fnMlClose)(hWnd)) ;
}

/*---- function MailInit ----------------------------------------------
 *
 *
 *
 */
int MailInit (HWND hWnd) 
{  
  int Mapi_Type;

  if (MailForceType >= 0)
    { /* Accept Private Profile directive to override Mail negotiation */
      MailCtrl.MailType = MailForceType;
    }
  else
    { /* MAPI is next on the negotiation list */
#ifdef WIN32
      if (GetVersion() & 0x80000000) /* win32s */
         Mapi_Type = 0;
      else
#endif
         Mapi_Type =  GetProfileInt("Mail","MAPI",0) ;

      if (Mapi_Type > 0)
   { 
     MailCtrl.MailType = MT_MAPI ;
   } else
   if (strcmp(SMTPHost,""))
   {  /* SMTP next on the negotiation list
         and assume that if SMTP host is specified use it */
     MailCtrl.MailType = MT_SMTP;
   } else 
   {
     MailCtrl.MailType = MT_NONE ;
   } ;
    }

  SetMailType (MailCtrl.MailType) ;

  (MailCtrl.fnMlInit)(hWnd);
      
  return (0);
}

/* -------------- MailCrashClose -----------------
 *
 *  On serious error in mail interface crash mail and
 * disable mail interface - winvn Ok to run without mail
 *
 * -----------------------------------------------*/

void
MailCrashExit(HWND hWnd)
{
  MailCtrl.fnMlClose(hWnd) ;  // clear down mail resources in use
  SetMailType(MT_NONE)  ;
}

int
mlSMTPClose(HWND hWnd)
{
	extern int close_smtp_socket();	/* in wvsckgen.c */
	
	return close_smtp_socket();
}

/*---- function MailxxxDefProc -------------------------------------------
 *
 *  Default Proc to support generic nature of dealing with multiple 
 *  mailing interfaces
 *                                                       
 */
MailDefCtlProc (HWND hWnd) {return (0);}
MailDefChrProc (HWND hWnd, TypTextBlock *textBlock, char *mBuf) {return (0);}
MailDefDocProc (HWND hWnd, TypDoc * Doc, int DocType) {return (0);}
MailDefDocProc2 (HWND hWnd, TypDoc * Doc) { return 0; }

/* --- SetMailType  ----------------------------------------------------
 *
 *  set up logic attributes for selected mail interface
 *
 */ 

void
SetMailType (int mailType)
{ 
#if 0
 static BOOL (*NoHeaderFuncs[])(TypDoc * Doc, char *Buf, long int BufLen) =
     {
      NULL
      };
 static BOOL (*MAPIHeaderFuncs[])(TypDoc * Doc, char *Buf, long int BufLen) =
     {
      GetArtHeaderTo, 
      GetArtHeaderSubject,
      NULL 
     };
 static BOOL (*SMTPHeaderFuncs[])(TypDoc * Doc, char *Buf, long int BufLen) =
     {
      MakeArtHeaderTo,
      MakeArtHeaderFrom,
      MakeArtHeaderSubject,
      MakeArtHeaderOrganization,
      MakeArtHeaderDate,
      MakeArtHeaderReplyTo,
      MakeArtHeaderXmailer,
      NULL
     };
#endif

  switch (mailType)
  {  
     case MT_MAPI :
   MailCtrl.enableMail = MF_ENABLED ;
   MailCtrl.enableLogout = MF_GRAYED ;
   MailCtrl.enableForward = MF_ENABLED ;
   MailCtrl.fnMlInit=mlMAPIInit ;
   MailCtrl.fnMlLogout=mlMAPILogout ;
   MailCtrl.fnMlClose=mlMAPIClose;
   MailCtrl.fnMlWinCreate=mlMAPISend ;
   break ;
      case MT_SMTP :
   MailCtrl.enableMail = MF_ENABLED ;
   MailCtrl.enableLogout = MF_GRAYED;
   MailCtrl.enableForward = MF_ENABLED ;   
   MailCtrl.fnMlInit=MailCtrl.fnMlLogout=MailDefCtlProc ;
   MailCtrl.fnMlLogout=MailDefCtlProc;
   MailCtrl.fnMlClose=mlSMTPClose;
   MailCtrl.fnMlWinCreate=(int (*)(HWND, TypDoc *, int))CreateComposeWnd;
   break;
   /* error handling for Fall through case message box plus */
   /* default to none */ 
     case MT_NONE :
     default:
   MailCtrl.enableMail = MF_GRAYED ;
   MailCtrl.enableLogout = MF_GRAYED;
   MailCtrl.enableForward = MF_GRAYED ;
   MailCtrl.fnMlClose=MailDefCtlProc;
   MailCtrl.fnMlInit=MailCtrl.fnMlLogout=MailDefCtlProc ;
//   MailCtrl.fnMlSend=MailDefChrProc ;
//   MailCtrl.fnMlForward=MailDefDocProc2;
   MailCtrl.fnMlWinCreate=MailDefDocProc;
//   MailCtrl.MailHeaderFuncs = NoHeaderFuncs ;
   break ;
  } ;
}

/* -------------- MailEnableLogout ------------------------
 *
 *  Allow discard (freeing) of mail resources during
 *  the session - Not necessarily available to all mail
 *  services
 *
 *--------------------------------------------------------*/  
void
MailEnableLogout(BOOL enable)
{
  if (enable) MailCtrl.enableLogout = MF_ENABLED ;
  else        MailCtrl.enableLogout = MF_GRAYED ;
}
